@@ -29,15 +29,15 @@
 /* clutter-actor.h */
 
 #include <glib-object.h>
-
-#include "clutter-fixed.h"
+#include <clutter/clutter-color.h>
+#include <clutter/clutter-fixed.h>
+#include <clutter/clutter-types.h>
+#include <clutter/clutter-units.h>
 
 G_BEGIN_DECLS
 
-#define CLUTTER_TYPE_GEOMETRY (clutter_geometry_get_type ())
-#define CLUTTER_TYPE_ACTOR_BOX (clutter_actor_box_get_type ())
-
-#define CLUTTER_TYPE_ACTOR clutter_actor_get_type()
+#define CLUTTER_TYPE_ACTOR_BOX  (clutter_actor_box_get_type ())
+#define CLUTTER_TYPE_ACTOR      (clutter_actor_get_type ())
 
 #define CLUTTER_ACTOR(obj) \
  (G_TYPE_CHECK_INSTANCE_CAST ((obj), CLUTTER_TYPE_ACTOR, ClutterActor))
@@ -62,39 +62,65 @@ typedef struct _ClutterActor         ClutterActor;
 typedef struct _ClutterActorClass    ClutterActorClass;
 typedef struct _ClutterActorBox      ClutterActorBox;
 typedef struct _ClutterActorPrivate  ClutterActorPrivate;
-typedef struct _ClutterGeometry      ClutterGeometry;
 
+/**
+ * ClutterCallback:
+ * @actor: a #ClutterActor
+ * @data: user data
+ *
+ * Generic callback
+ */
 typedef void (*ClutterCallback) (ClutterActor *actor, gpointer data);
 #define CLUTTER_CALLBACK(f)	((ClutterCallback) (f))
 
-struct _ClutterGeometry
-{
-  /* FIXME: 
-   * It is likely gonna save a load of pain if we make 
-   * x,y unsigned...
-  */
-  gint x;
-  gint y;
-  guint width;
-  guint height;
-};
-
-GType clutter_geometry_get_type (void) G_GNUC_CONST;
-
+/**
+ * ClutterActorFlags:
+ * @CLUTTER_ACTOR_MAPPED: the actor has been painted
+ * @CLUTTER_ACTOR_REALIZED: the resources associated to the actor have been
+ *   allocated
+ *
+ * Flags used to signal the state of an actor.
+ */
 typedef enum
 {
   CLUTTER_ACTOR_MAPPED   = 1 << 1,
   CLUTTER_ACTOR_REALIZED = 1 << 2
 } ClutterActorFlags;
 
-struct _ClutterActorBox { gint x1, y1, x2, y2; };
+/**
+ * ClutterActorBox:
+ * @x1: X coordinate of the top left corner
+ * @y1: Y coordinate of the top left corner
+ * @x2: X coordinate of the bottom right corner
+ * @y2: Y coordinate of the bottom right corner
+ *
+ * Bounding box of an actor. The coordinates of the top left and right bottom
+ * corners of an actor. The coordinates of the two points are expressed in
+ * #ClutterUnit<!-- -->s, that is are device-independent. If you want to obtain
+ * the box dimensions in pixels, use clutter_actor_get_geometry().
+ */
+struct _ClutterActorBox
+{
+  ClutterUnit x1;
+  ClutterUnit y1;
+  ClutterUnit x2;
+  ClutterUnit y2;
+};
 
 GType clutter_actor_box_get_type (void) G_GNUC_CONST;
 
+/**
+ * ClutterActor:
+ * @flags: #ClutterActorFlags
+ *
+ * Base class for actors.
+ */
 struct _ClutterActor
 {
+  /*< private >*/
+  GInitiallyUnowned parent_instance;
+  
   /*< public >*/
-  GObject parent_instance;
   guint32 flags;
 
   /*< private >*/
@@ -103,39 +129,66 @@ struct _ClutterActor
   ClutterActorPrivate *priv;
 };
 
+/**
+ * ClutterActorClass:
+ * @show: signal class handler for the ClutterActor::show signal
+ * @show_all: virtual function for containers and composite actors, to
+ *   determine which children should be shown when calling
+ *   clutter_actor_show_all() on the actor. Defaults to calling
+ *   clutter_actor_show().
+ * @hide: signal class handler for the ClutterActor::hide signal
+ * @hide_all: virtual function for containers and composite actors, to
+ *   determine which children should be shown when calling
+ *   clutter_actor_hide_all() on the actor. Defaults to calling
+ *   clutter_actor_show().
+ * @realize: virtual function, used to allocate resources for the actor;
+ *   it should chain up to the parent's implementation
+ * @unrealize: virtual function, used to deallocate resources allocated
+ *   in ::realized; it should chain up to the parent's implementation
+ * @paint: virtual function, used to paint the actor
+ * @request_coords: virtual function, used when setting the coordinates
+ *   of an actor
+ * @query_coords: virtual function, used when querying the actor for
+ *   its coordinates
+ * @set_depth: virtual function, used when setting the depth
+ * @get_depth: virtual function, used when getting the depth
+ * @parent_set: signal class closure for the ClutterActor::parent-set
+ *   signal
+ * @destroy: signal class closure for the ClutterActor::destroy signal
+ * @pick: virtual functions, used to draw an outline of the actor
+ *
+ * Base class for actors.
+ */
 struct _ClutterActorClass
 {
-  GObjectClass parent_class;
+  /*< private >*/
+  GInitiallyUnownedClass parent_class;
 
-  void (* show)            (ClutterActor    *actor);
-  void (* show_all)        (ClutterActor    *actor);
-  void (* hide)            (ClutterActor    *actor);
-  void (* hide_all)        (ClutterActor    *actor);
-  void (* realize)         (ClutterActor    *actor);
-  void (* unrealize)       (ClutterActor    *actor);
-  void (* paint)           (ClutterActor    *actor);
-  void (* request_coords)  (ClutterActor    *actor,
-			    ClutterActorBox *box);
-  void (* allocate_coords) (ClutterActor    *actor,
-			    ClutterActorBox *box);
-  void (* set_depth)       (ClutterActor    *actor,
-		            gint             depth);
-  gint (* get_depth)       (ClutterActor    *actor);
-  void (* parent_set)      (ClutterActor    *actor,
-                            ClutterActor    *old_parent);
+  /*< public >*/
+  void (* show)            (ClutterActor        *actor);
+  void (* show_all)        (ClutterActor        *actor);
+  void (* hide)            (ClutterActor        *actor);
+  void (* hide_all)        (ClutterActor        *actor);
+  void (* realize)         (ClutterActor        *actor);
+  void (* unrealize)       (ClutterActor        *actor);
+  void (* paint)           (ClutterActor        *actor);
+  void (* request_coords)  (ClutterActor        *actor,
+			    ClutterActorBox     *box);
+  void (* query_coords)    (ClutterActor        *actor,
+			    ClutterActorBox     *box);
+  void (* set_depth)       (ClutterActor        *actor,
+		            gint                 depth);
+  gint (* get_depth)       (ClutterActor        *actor);
+  void (* parent_set)      (ClutterActor        *actor,
+                            ClutterActor        *old_parent);
 
-  void (* destroy)         (ClutterActor    *actor);
-
-  /* to go ? */
-  void (* queue_redraw)    (ClutterActor    *actor);
+  void (* destroy)         (ClutterActor        *actor);
+  void (* pick)            (ClutterActor        *actor,
+                            const ClutterColor  *color);
 
+  /*< private >*/
   /* padding for future expansion */
-  void (*_clutter_actor_1) (void);
-  void (*_clutter_actor_2) (void);
-  void (*_clutter_actor_3) (void);
-  void (*_clutter_actor_4) (void);
-  void (*_clutter_actor_5) (void);
-  void (*_clutter_actor_6) (void);
+  gpointer _padding_dummy[32];
 };
 
 GType                 clutter_actor_get_type         (void) G_GNUC_CONST;
@@ -146,21 +199,23 @@ void                  clutter_actor_hide_all         (ClutterActor          *sel
 void                  clutter_actor_realize          (ClutterActor          *self);
 void                  clutter_actor_unrealize        (ClutterActor          *self);
 void                  clutter_actor_paint            (ClutterActor          *self);
+void                  clutter_actor_pick             (ClutterActor          *self, 
+						      const ClutterColor    *color);
 void                  clutter_actor_queue_redraw     (ClutterActor          *self);
 void                  clutter_actor_destroy          (ClutterActor          *self);
 void                  clutter_actor_request_coords   (ClutterActor          *self,
 						      ClutterActorBox       *box);
-void                  clutter_actor_allocate_coords  (ClutterActor          *self,
+void                  clutter_actor_query_coords     (ClutterActor          *self,
 						      ClutterActorBox       *box);
 void                  clutter_actor_set_geometry     (ClutterActor          *self,
 						      const ClutterGeometry *geometry);
 void                  clutter_actor_get_geometry     (ClutterActor          *self,
 						      ClutterGeometry       *geometry);
 void                  clutter_actor_get_coords       (ClutterActor          *self,
-						      gint                  *x1,
-						      gint                  *y1,
-						      gint                  *x2,
-						      gint                  *y2);
+						      gint                  *x_1,
+						      gint                  *y_1,
+						      gint                  *x_2,
+						      gint                  *y_2);
 void                  clutter_actor_set_size         (ClutterActor          *self,
 						      gint                   width,
 						      gint                   height);
@@ -194,6 +249,24 @@ void                  clutter_actor_rotate_z         (ClutterActor          *sel
 						      gfloat                 angle,
 						      gint                   x,
 						      gint                   y);
+void                  clutter_actor_rotate_xx        (ClutterActor          *self,
+						      ClutterFixed           angle,
+						      gint                   y,
+						      gint                   z);
+void                  clutter_actor_rotate_yx        (ClutterActor          *self,
+						      ClutterFixed           angle,
+						      gint                   x,
+						      gint                   z);
+void                  clutter_actor_rotate_zx        (ClutterActor          *self,
+						      ClutterFixed           angle,
+						      gint                   x,
+						      gint                   y);
+gdouble               clutter_actor_get_rxang        (ClutterActor          *self);
+gdouble               clutter_actor_get_ryang        (ClutterActor          *self);
+gdouble               clutter_actor_get_rzang        (ClutterActor          *self);
+ClutterFixed          clutter_actor_get_rxangx       (ClutterActor          *self);
+ClutterFixed          clutter_actor_get_ryangx       (ClutterActor          *self);
+ClutterFixed          clutter_actor_get_rzangx       (ClutterActor          *self);
 void                  clutter_actor_set_opacity      (ClutterActor          *self,
 						      guint8                 opacity);
 guint8                clutter_actor_get_opacity      (ClutterActor          *self);
@@ -235,16 +308,34 @@ void                  clutter_actor_get_scalex       (ClutterActor          *sel
 void                  clutter_actor_get_scale        (ClutterActor          *self,
                                                       gdouble               *scale_x,
                                                       gdouble               *scale_y);
-void                  clutter_actor_get_abs_size     (ClutterActor          *self,
-                                                      guint                 *width,
-                                                      guint                 *height);
-void                  clutter_actor_get_size         (ClutterActor          *self,
-                                                      guint                 *width,
-                                                      guint                 *height);
-void                  clutter_actor_move_by          (ClutterActor          *self,
-                                                      gint                   dx,
-                                                      gint                   dy);
 
+void                  clutter_actor_set_scale_with_gravityx  (ClutterActor          *self,
+							      ClutterFixed      scale_x,
+							      ClutterFixed      scale_y,
+							      ClutterGravity    gravity);
+
+void                  clutter_actor_set_scale_with_gravity   (ClutterActor          *self,
+							      gfloat                 scale_x,
+							      gfloat                 scale_y,
+							      ClutterGravity         gravity);
+
+void                  clutter_actor_get_abs_size             (ClutterActor          *self,
+                                                              guint                 *width,
+                                                              guint                 *height);
+void                  clutter_actor_get_size                 (ClutterActor          *self,
+                                                              guint                 *width,
+                                                              guint                 *height);
+void                  clutter_actor_move_by                  (ClutterActor          *self,
+                                                              gint                   dx,
+                                                              gint                   dy);
+
+void                  clutter_actor_get_vertices             (ClutterActor          *self,
+						              ClutterVertex          verts[4]);
+
+void                  clutter_actor_apply_transform_to_point (ClutterActor          *self, 
+						              ClutterVertex         *point,
+							      ClutterVertex         *vertex);
+     
 G_END_DECLS
 
 #endif /* _HAVE_CLUTTER_ACTOR_H */
