@@ -6,8 +6,9 @@
  * Authored By Matthew Allum  <mallum@openedhand.com>
  *             Jorn Baayen  <jorn@openedhand.com>
  *             Emmanuele Bassi  <ebassi@openedhand.com>
+ *             Tomas Frydrych <tf@openedhand.com>
  *
- * Copyright (C) 2006 OpenedHand
+ * Copyright (C) 2006, 2007 OpenedHand
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -499,7 +500,7 @@ sincx1024_func (ClutterAlpha *alpha,
   ClutterTimeline *timeline;
   gint current_frame_num, n_frames;
   ClutterAngle x;
-  ClutterFixed sine;
+  unsigned int sine;
   
   timeline = clutter_alpha_get_timeline (alpha);
 
@@ -508,18 +509,14 @@ sincx1024_func (ClutterAlpha *alpha,
 
   x = angle * current_frame_num / n_frames;
 
-  /*
-   * x = x*PI - PI/angle
-   */
-  x = (x << 9) - (512 / angle);
+  x -= (512 * 512 / angle);
+  
+  sine = ((clutter_sini (x) + offset)/2) * CLUTTER_ALPHA_MAX_ALPHA;
 
-  sine = (clutter_sini (x) + offset)/2;
-
-  CLUTTER_NOTE (ALPHA, "sine: %2f\n", CLUTTER_FIXED_TO_DOUBLE (sine));
-
-  return CLUTTER_FIXED_INT (sine * CLUTTER_ALPHA_MAX_ALPHA);
+  sine = sine >> CFX_Q;
+  
+  return sine;
 }
-
 #if 0
 /*
  * The following two functions are left in place for reference
@@ -549,6 +546,9 @@ sincx_func (ClutterAlpha *alpha,
   return CLUTTER_FIXED_INT (sine * CLUTTER_ALPHA_MAX_ALPHA);
 }
 
+/* NB: angle is not in radians but in muliples of PI, i.e., 2.0
+ * represents full circle.
+ */
 static guint32
 sinc_func (ClutterAlpha *alpha, 
 	   float         angle,
@@ -568,11 +568,12 @@ sinc_func (ClutterAlpha *alpha,
   x = (gdouble) (current_frame_num * angle * M_PI) / n_frames ;
   sine = (sin (x - (M_PI / angle)) + offset) * 0.5f;
 
-  CLUTTER_NOTE (ALPHA, "sine: %2f\n", sine);
+  CLUTTER_NOTE (ALPHA, "sine: %2f\n",sine);
 
-  return (guint32) (sine * (gdouble) CLUTTER_ALPHA_MAX_ALPHA);
+  return CLUTTER_FLOAT_TO_INT ((sine * (gdouble) CLUTTER_ALPHA_MAX_ALPHA));
 }
 #endif
+
 /**
  * clutter_sine_func:
  * @alpha: a #ClutterAlpha
@@ -590,13 +591,10 @@ clutter_sine_func (ClutterAlpha *alpha,
                    gpointer      dummy)
 {
 #if 0
-  return sinc_func (alpha, 2.0, 1.0);
+    return sinc_func (alpha, 2.0, 1.0);
 #else
-  /* Conversion to ClutterAngle:
-   * 
-   * 2.0 rad -> 1024*2.0/(2*PI) = 325.949
-   */
-  return sincx1024_func (alpha, 326, CFX_ONE);
+    /* 2.0 above represents full circle */
+    return sincx1024_func (alpha, 1024, CFX_ONE);
 #endif
 }
 
@@ -605,8 +603,9 @@ clutter_sine_func (ClutterAlpha *alpha,
  * @alpha: a #ClutterAlpha
  * @dummy: unused argument
  *
- * Convenience alpha function for a sine wave. You can use this
- * function as the alpha function for clutter_alpha_set_func().
+ * Convenience alpha function for a sine wave over interval [0, pi / 2].
+ * You can use this function as the alpha function for
+ * clutter_alpha_set_func().
  *
  * Return value: an alpha value.
  *
@@ -616,13 +615,269 @@ guint32
 clutter_sine_inc_func (ClutterAlpha *alpha,
 		       gpointer      dummy)
 {
-#if 0
-  return sinc_func (alpha, 0.5, 1.0);
-#else
-  /* Conversion to ClutterAngle:
+  ClutterTimeline * timeline;
+  gint              frame;
+  gint              n_frames;
+  ClutterAngle      x;
+  ClutterFixed      sine;
+  
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  x = 256 * frame / n_frames;
+
+  sine = clutter_sini (x) * CLUTTER_ALPHA_MAX_ALPHA;
+
+  return ((guint32)sine) >> CFX_Q;
+}
+
+/**
+ * clutter_sine_dec_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused argument
+ *
+ * Convenience alpha function for a sine wave over interval [pi / 2, pi].
+ * You can use this function as the alpha function for
+ * clutter_alpha_set_func().
+ *
+ * Return value: an alpha value.
+ *
+ * Since: 0.4
+ */
+guint32 
+clutter_sine_dec_func (ClutterAlpha *alpha,
+		       gpointer      dummy)
+{
+  ClutterTimeline * timeline;
+  gint              frame;
+  gint              n_frames;
+  ClutterAngle      x;
+  ClutterFixed      sine;
+  
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  x = 256 * frame / n_frames + 256;
+
+  sine = clutter_sini (x) * CLUTTER_ALPHA_MAX_ALPHA;
+
+  return ((guint32)sine) >> CFX_Q;
+}
+
+/**
+ * clutter_sine_half_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused argument
+ *
+ * Convenience alpha function for a sine wave over interval [0, pi].
+ * You can use this function as the alpha function for
+ * clutter_alpha_set_func().
+ *
+ * Return value: an alpha value.
+ *
+ * Since: 0.4
+ */
+guint32 
+clutter_sine_half_func (ClutterAlpha *alpha,
+			gpointer      dummy)
+{
+  ClutterTimeline * timeline;
+  gint              frame;
+  gint              n_frames;
+  ClutterAngle      x;
+  ClutterFixed      sine;
+  
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  x = 512 * frame / n_frames;
+
+  sine = clutter_sini (x) * CLUTTER_ALPHA_MAX_ALPHA;
+
+  return ((guint32)sine) >> CFX_Q;
+}
+
+/**
+ * clutter_square_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused argument
+ *
+ * Convenience alpha function for a square wave. You can use this
+ * function as the alpha function for clutter_alpha_set_func().
+ *
+ * Return value: an alpha value
+ *
+ * Since: 0.4
+ */
+guint32
+clutter_square_func (ClutterAlpha *alpha,
+                     gpointer      dummy)
+{
+  ClutterTimeline *timeline;
+  gint current_frame_num, n_frames;
+
+  timeline = clutter_alpha_get_timeline (alpha);
+
+  current_frame_num = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  return (current_frame_num > (n_frames / 2)) ? CLUTTER_ALPHA_MAX_ALPHA
+                                              : 0;
+}
+
+
+/**
+ * clutter_smoothstep_inc_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused
+ *
+ * Convenience alpha function for a smoothstep curve. You can use this
+ * function as the alpha function for clutter_alpha_set_func().
+ *
+ * Return value: an alpha value
+ *
+ * Since: 0.4
+ */
+guint32
+clutter_smoothstep_inc_func (ClutterAlpha  *alpha,
+			     gpointer       dummy)
+{
+  ClutterTimeline    *timeline;
+  gint                frame;
+  gint                n_frames;
+  guint32             r; 
+  guint32             x; 
+
+  /*
+   * The smoothstep function uses f(x) = -2x^3 + 3x^2 where x is from <0,1>,
+   * and precission is critical -- we use 8.24 fixed format for this operation.
+   * The earlier operations involve division, which we cannot do in 8.24 for
+   * numbers in <0,1> we use ClutterFixed.
+   */
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  /*
+   * Convert x to 8.24 for next step.
+   */
+  x = CFX_DIV (frame, n_frames) << 8;
+
+  /*
+   * f(x) = -2x^3 + 3x^2
+   * 
+   * Convert result to ClutterFixed to avoid overflow in next step.
+   */
+  r = ((x >> 12) * (x >> 12) * 3 - (x >> 15) * (x >> 16) * (x >> 16)) >> 8;
+
+  return CFX_INT (r * CLUTTER_ALPHA_MAX_ALPHA);
+}
+
+/**
+ * clutter_smoothstep_dec_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused
+ *
+ * Convenience alpha function for a downward smoothstep curve. You can use
+ * this function as the alpha function for clutter_alpha_set_func().
+ *
+ * Return value: an alpha value
+ *
+ * Since: 0.4
+ */
+guint32
+clutter_smoothstep_dec_func (ClutterAlpha  *alpha,
+			     gpointer       dummy)
+{
+  return CLUTTER_ALPHA_MAX_ALPHA - clutter_smoothstep_inc_func (alpha, dummy);
+}
+
+/**
+ * clutter_exp_inc_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused argument
+ *
+ * Convenience alpha function for a 2^x curve. You can use this function as the
+ * alpha function for clutter_alpha_set_func().
+ *
+ * Return value: an alpha value.
+ *
+ * Since: 0.4
+ */
+guint32 
+clutter_exp_inc_func (ClutterAlpha *alpha,
+		      gpointer      dummy)
+{
+  ClutterTimeline * timeline;
+  gint              frame;
+  gint              n_frames;
+  ClutterFixed      x;
+  ClutterFixed      x_alpha_max = 0x100000;
+  guint32           result;
+  
+  /*
+   * Choose x_alpha_max such that
+   * 
+   *   (2^x_alpha_max) - 1 == CLUTTER_ALPHA_MAX_ALPHA
+   */
+#if CLUTTER_ALPHA_MAX_ALPHA != 0xffff
+#error Adjust x_alpha_max to match CLUTTER_ALPHA_MAX_ALPHA
+#endif
+  
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  x =  x_alpha_max * frame / n_frames;
+
+  result = clutter_pow2x (x) - 1;
+
+  return result;
+}
+
+
+/**
+ * clutter_exp_dec_func:
+ * @alpha: a #ClutterAlpha
+ * @dummy: unused argument
+ *
+ * Convenience alpha function for a decreasing 2^x curve. You can use this
+ * function as the alpha function for clutter_alpha_set_func().
+ *
+ * Return value: an alpha value.
+ *
+ * Since: 0.4
+ */
+guint32 
+clutter_exp_dec_func (ClutterAlpha *alpha,
+		      gpointer      dummy)
+{
+  ClutterTimeline * timeline;
+  gint              frame;
+  gint              n_frames;
+  ClutterFixed      x;
+  ClutterFixed      x_alpha_max = 0x100000;
+  guint32           result;
+  
+  /*
+   * Choose x_alpha_max such that
    * 
-   * 0.5 rad -> 1024*0.5/(2*PI) = 81.487
+   *   (2^x_alpha_max) - 1 == CLUTTER_ALPHA_MAX_ALPHA
    */
-  return sincx1024_func (alpha, 81, CFX_ONE);
+#if CLUTTER_ALPHA_MAX_ALPHA != 0xffff
+#error Adjust x_alpha_max to match CLUTTER_ALPHA_MAX_ALPHA
 #endif
+  
+  timeline = clutter_alpha_get_timeline (alpha);
+  frame    = clutter_timeline_get_current_frame (timeline);
+  n_frames = clutter_timeline_get_n_frames (timeline);
+
+  x =  (x_alpha_max * (n_frames - frame)) / n_frames;
+
+  result = clutter_pow2x (x) - 1;
+
+  return result;
 }
