@@ -58,6 +58,86 @@
 #include "clutter-behaviour.h"
 #include "clutter-debug.h"
 #include "clutter-private.h"
+#include "clutter-marshal.h"
+
+/**
+ * clutter_knot_copy:
+ * @knot: a #ClutterKnot
+ *
+ * Makes an allocated copy of a knot.
+ *
+ * Return value: the copied knot.
+ *
+ * Since: 0.2
+ */
+ClutterKnot *
+clutter_knot_copy (const ClutterKnot *knot)
+{
+  ClutterKnot *copy;
+
+  copy = g_slice_new0 (ClutterKnot);
+  
+  *copy = *knot;
+
+  return copy;
+}
+
+/**
+ * clutter_knot_free:
+ * @knot: a #ClutterKnot
+ *
+ * Frees the memory of an allocated knot.
+ *
+ * Since: 0.2
+ */
+void
+clutter_knot_free (ClutterKnot *knot)
+{
+  if (G_LIKELY (knot))
+    {
+      g_slice_free (ClutterKnot, knot);
+    }
+}
+
+/**
+ * clutter_knot_equal:
+ * @knot_a: First knot
+ * @knot_b: Second knot
+ *
+ * Compares to knot and checks if the point to the same location.
+ *
+ * Return value: %TRUE if the knots point to the same location.
+ *
+ * Since: 0.2
+ */
+gboolean
+clutter_knot_equal (const ClutterKnot *knot_a,
+                    const ClutterKnot *knot_b)
+{
+  g_return_val_if_fail (knot_a != NULL, FALSE);
+  g_return_val_if_fail (knot_b != NULL, FALSE);
+
+  if (knot_a == knot_b)
+    return TRUE;
+
+  return knot_a->x == knot_b->x && knot_a->y == knot_b->y;
+}
+
+GType
+clutter_knot_get_type (void)
+{
+  static GType our_type = 0;
+
+  if (G_UNLIKELY (!our_type))
+    {
+      our_type =
+        g_boxed_type_register_static ("ClutterKnot",
+                                      (GBoxedCopyFunc) clutter_knot_copy,
+                                      (GBoxedFreeFunc) clutter_knot_free);
+    }
+
+  return our_type;
+}
 
 G_DEFINE_ABSTRACT_TYPE (ClutterBehaviour,
                         clutter_behaviour,
@@ -78,26 +158,27 @@ enum
 };
 
 enum {
-  SIGNAL_LAST
+  APPLIED,
+  REMOVED,
+  LAST_SIGNAL
 };
 
+static guint behave_signals[LAST_SIGNAL] = { 0 };
 
 #define CLUTTER_BEHAVIOUR_GET_PRIVATE(obj)         \
               (G_TYPE_INSTANCE_GET_PRIVATE ((obj), \
                CLUTTER_TYPE_BEHAVIOUR,             \
                ClutterBehaviourPrivate))
 
-static void 
-clutter_behaviour_finalize (GObject *object)
+static void
+clutter_behaviour_dispose (GObject *gobject)
 {
-  ClutterBehaviour *self = CLUTTER_BEHAVIOUR (object);
+  ClutterBehaviour *self = CLUTTER_BEHAVIOUR (gobject);
 
   clutter_behaviour_set_alpha (self, NULL);
-  
-  g_slist_foreach (self->priv->actors, (GFunc) g_object_unref, NULL);
-  g_slist_free (self->priv->actors);
+  clutter_behaviour_remove_all (self);
 
-  G_OBJECT_CLASS (clutter_behaviour_parent_class)->finalize (object);
+  G_OBJECT_CLASS (clutter_behaviour_parent_class)->dispose (gobject);
 }
 
 static void
@@ -106,9 +187,7 @@ clutter_behaviour_set_property (GObject      *object,
 				const GValue *value, 
 				GParamSpec   *pspec)
 {
-  ClutterBehaviour *behaviour;
-
-  behaviour = CLUTTER_BEHAVIOUR(object);
+  ClutterBehaviour *behaviour = CLUTTER_BEHAVIOUR (object);
 
   switch (prop_id) 
     {
@@ -127,11 +206,8 @@ clutter_behaviour_get_property (GObject    *object,
 			        GValue     *value, 
 			        GParamSpec *pspec)
 {
-  ClutterBehaviour        *behaviour;
-  ClutterBehaviourPrivate *priv;
-
-  behaviour = CLUTTER_BEHAVIOUR(object);
-  priv      = CLUTTER_BEHAVIOUR_GET_PRIVATE(behaviour);
+  ClutterBehaviour        *behaviour = CLUTTER_BEHAVIOUR (object);
+  ClutterBehaviourPrivate *priv = behaviour->priv;
 
   switch (prop_id) 
     {
@@ -157,21 +233,68 @@ clutter_behaviour_class_init (ClutterBehaviourClass *klass)
 {
   GObjectClass *object_class = G_OBJECT_CLASS (klass);
 
-  object_class->finalize     = clutter_behaviour_finalize;
+  object_class->dispose      = clutter_behaviour_dispose;
   object_class->set_property = clutter_behaviour_set_property;
   object_class->get_property = clutter_behaviour_get_property;
 
+  /**
+   * ClutterBehaviour:alpha:
+   *
+   * The #ClutterAlpha object used to drive this behaviour. A #ClutterAlpha
+   * object binds a #ClutterTimeline and a function which computes a value
+   * (the "alpha") depending on the time. Each time the alpha value changes
+   * the alpha-notify virtual function is called.
+   *
+   * Since: 0.2
+   */
   g_object_class_install_property (object_class,
                                    PROP_ALPHA,
                                    g_param_spec_object ("alpha",
                                                         "Alpha",
                                                         "Alpha Object to drive the behaviour",
                                                         CLUTTER_TYPE_ALPHA,
-                                                        G_PARAM_CONSTRUCT |
                                                         CLUTTER_PARAM_READWRITE));
 
   klass->alpha_notify = clutter_behaviour_alpha_notify_unimplemented;
 
+  /**
+   * ClutterBehaviour::applied:
+   * @behaviour: the #ClutterBehaviour that received the signal
+   * @actor: the actor the behaviour was applied to.
+   *
+   * The ::apply signal is emitted each time the behaviour is applied
+   * to an actor.
+   *
+   * Since: 0.4
+   */
+  behave_signals[APPLIED] =
+    g_signal_new ("applied",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (ClutterBehaviourClass, applied),
+		  NULL, NULL,
+		  clutter_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  CLUTTER_TYPE_ACTOR);
+  /**
+   * ClutterBehaviour::removed:
+   * @behaviour: the #ClutterBehaviour that received the signal
+   * @actor: the removed actor
+   *
+   * The ::removed signal is emitted each time a behaviour is not applied
+   * to an actor anymore.
+   *
+   */
+  behave_signals[REMOVED] =
+    g_signal_new ("removed",
+		  G_OBJECT_CLASS_TYPE (object_class),
+		  G_SIGNAL_RUN_FIRST,
+		  G_STRUCT_OFFSET (ClutterBehaviourClass, removed),
+		  NULL, NULL,
+		  clutter_marshal_VOID__OBJECT,
+		  G_TYPE_NONE, 1,
+		  CLUTTER_TYPE_ACTOR);
+
   g_type_class_add_private (klass, sizeof (ClutterBehaviourPrivate));
 }
 
@@ -198,10 +321,14 @@ void
 clutter_behaviour_apply (ClutterBehaviour *behave,
                          ClutterActor     *actor)
 {
+  ClutterBehaviourPrivate *priv;
+
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR (behave));
   g_return_if_fail (CLUTTER_IS_ACTOR (actor));
 
-  if (g_slist_find (behave->priv->actors, actor))
+  priv = behave->priv;
+
+  if (g_slist_find (priv->actors, actor))
     {
       g_warning ("The behaviour of type %s already applies "
                  "to the actor of type %s",
@@ -210,8 +337,30 @@ clutter_behaviour_apply (ClutterBehaviour *behave,
       return;
     }
 
-  g_object_ref (actor);
-  behave->priv->actors = g_slist_prepend (behave->priv->actors, actor);
+  priv->actors = g_slist_prepend (priv->actors, g_object_ref (actor));
+
+  g_signal_emit (behave, behave_signals[APPLIED], 0, actor);
+}
+
+/**
+ * clutter_behaviour_is_applied:
+ * @behave: a #ClutterBehaviour
+ * @actor: a #ClutterActor
+ *
+ * Check if @behave applied to  @actor.
+ *
+ * Return value: TRUE if actor has behaviour. FALSE otherwise.
+ *
+ * Since: 0.4
+ */
+gboolean
+clutter_behaviour_is_applied (ClutterBehaviour *behave,
+			      ClutterActor     *actor)
+{
+  g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR (behave), FALSE);
+  g_return_val_if_fail (CLUTTER_IS_ACTOR (actor), FALSE);
+
+  return (g_slist_find (behave->priv->actors, actor) != NULL);
 }
 
 /**
@@ -228,10 +377,14 @@ void
 clutter_behaviour_remove (ClutterBehaviour *behave,
                           ClutterActor     *actor)
 {
+  ClutterBehaviourPrivate *priv;
+
   g_return_if_fail (CLUTTER_IS_BEHAVIOUR (behave));
   g_return_if_fail (CLUTTER_IS_ACTOR (actor));
 
-  if (!g_slist_find (behave->priv->actors, actor))
+  priv = behave->priv;
+
+  if (!g_slist_find (priv->actors, actor))
     {
       g_warning ("The behaviour of type %s is not applied "
                  "to the actor of type %s",
@@ -240,8 +393,11 @@ clutter_behaviour_remove (ClutterBehaviour *behave,
       return;
     }
   
+  priv->actors = g_slist_remove (priv->actors, actor);
+  
+  g_signal_emit (behave, behave_signals[REMOVED], 0, actor);
+
   g_object_unref (actor);
-  behave->priv->actors = g_slist_remove (behave->priv->actors, actor);
 }
 
 /**
@@ -265,21 +421,21 @@ clutter_behaviour_get_n_actors (ClutterBehaviour *behave)
 /**
  * clutter_behaviour_get_nth_actor:
  * @behave: a #ClutterBehaviour
- * @index:  the index of an actor this behaviour is applied too. 
+ * @index_: the index of an actor this behaviour is applied too. 
  *
  * Gets an actor the behaviour was applied to referenced by index num.
  *
- * Return value: A Clutter actor or NULL if index is invalid.
+ * Return value: A Clutter actor or NULL if @index_ is invalid.
  *
  * Since: 0.2
  */
 ClutterActor*
 clutter_behaviour_get_nth_actor (ClutterBehaviour *behave,
-				 gint              index)
+				 gint              index_)
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR (behave), NULL);
 
-  return g_slist_nth_data (behave->priv->actors, index);
+  return g_slist_nth_data (behave->priv->actors, index_);
 }
 
 
@@ -324,7 +480,7 @@ clutter_behaviour_actors_foreach (ClutterBehaviour            *behave,
  * 
  * Since: 0.2
  */
-ClutterAlpha*
+ClutterAlpha *
 clutter_behaviour_get_alpha (ClutterBehaviour *behave)
 {
   g_return_val_if_fail (CLUTTER_IS_BEHAVIOUR (behave), NULL);
@@ -418,7 +574,7 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
  * @behave: a #ClutterBehaviour
  *
  * Retrieves all the actors to which @behave applies. It is not recommended
- * derived classes use this in there alpha notify method but use 
+ * for derived classes to use this in there alpha notify method but use 
  * #clutter_behaviour_actors_foreach as it avoids alot of needless allocations.
  *
  * Return value: a list of actors. You should free the returned list
@@ -429,13 +585,44 @@ clutter_behaviour_set_alpha (ClutterBehaviour *behave,
 GSList *
 clutter_behaviour_get_actors (ClutterBehaviour *behave)
 {
+  ClutterBehaviourPrivate *priv;
   GSList *retval, *l;
 
   g_return_val_if_fail (CLUTTER_BEHAVIOUR (behave), NULL);
 
+  priv = behave->priv;
   retval = NULL;
-  for (l = behave->priv->actors; l != NULL; l = l->next)
+  for (l = priv->actors; l != NULL; l = l->next)
     retval = g_slist_prepend (retval, l->data);
 
-  return g_slist_reverse (retval);
+  return retval;
+}
+
+/**
+ * clutter_behaviour_remove_all:
+ * @behave: a #ClutterBehaviour
+ *
+ * Removes every actor from the list that @behave holds.
+ *
+ * Since: 0.4
+ */
+void
+clutter_behaviour_remove_all (ClutterBehaviour *behave)
+{
+  ClutterBehaviourPrivate *priv;
+  GSList *l;
+
+  g_return_if_fail (CLUTTER_IS_BEHAVIOUR (behave));
+
+  priv = behave->priv;
+  for (l = priv->actors; l != NULL; l = l->next)
+    {
+      ClutterActor *actor = l->data;
+
+      g_signal_emit (behave, behave_signals[REMOVED], 0, actor);
+      g_object_unref (actor);
+    }
+
+  g_slist_free (priv->actors);
+  priv->actors = NULL;
 }
