@@ -41,6 +41,8 @@
 #include "clutter-private.h"
 #include "clutter-debug.h"
 
+#include "cogl.h"
+
 enum
 {
   PROP_0,
@@ -61,14 +63,14 @@ struct _ClutterCloneTexturePrivate
 
 static void
 clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture, 
-				 int             x1, 
-				 int             y1, 
-				 int             x2, 
-				 int             y2)
+				 int                  x_1, 
+				 int                  y_1, 
+				 int                  x_2, 
+				 int                  y_2)
 {
   gint   qx1 = 0, qx2 = 0, qy1 = 0, qy2 = 0;
   gint   qwidth = 0, qheight = 0;
-  gint   x, y, i =0, lastx = 0, lasty = 0;
+  gint   x, y, i = 0, lastx = 0, lasty = 0;
   gint   n_x_tiles, n_y_tiles; 
   gint   pwidth, pheight;
   float tx, ty;
@@ -78,8 +80,8 @@ clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture,
 
   priv = ctexture->priv;
 
-  qwidth  = x2 - x1;
-  qheight = y2 - y1;
+  qwidth  = x_2 - x_1;
+  qheight = y_2 - y_1;
 
   if (!CLUTTER_ACTOR_IS_REALIZED (parent_actor))
       clutter_actor_realize (parent_actor);
@@ -107,16 +109,11 @@ clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture,
 	  ty = (float) pheight / clutter_util_next_p2 (pheight);
 	}
 
-      qx1 = x1; qx2 = x2;
-      qy1 = y1; qy2 = y2;
-      
-      glBegin (GL_QUADS);
-      glTexCoord2f (tx, ty);   glVertex2i   (qx2, qy2);
-      glTexCoord2f (0,  ty);   glVertex2i   (qx1, qy2);
-      glTexCoord2f (0,  0);    glVertex2i   (qx1, qy1);
-      glTexCoord2f (tx, 0);    glVertex2i   (qx2, qy1);
-      glEnd ();	
-      
+      cogl_texture_quad (x_1, x_2, y_1, y_2, 
+			 0,
+			 0,
+			 CLUTTER_FLOAT_TO_FIXED (tx),
+			 CLUTTER_FLOAT_TO_FIXED (ty));
       return;
     }
 
@@ -145,10 +142,10 @@ clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture,
 	  tx = (float) actual_w / xsize;
 	  ty = (float) actual_h / ysize;
 
-	  qx1 = x1 + lastx;
+	  qx1 = x_1 + lastx;
 	  qx2 = qx1 + ((qwidth * actual_w ) / pwidth );
 	  
-	  qy1 = y1 + lasty;
+	  qy1 = y_1 + lasty;
 	  qy2 = qy1 + ((qheight * actual_h) / pheight );
 
 	  CLUTTER_NOTE (TEXTURE,
@@ -156,12 +153,11 @@ clone_texture_render_to_gl_quad (ClutterCloneTexture *ctexture,
 			x, y,
 			actual_w, actual_h);
 
-	  glBegin (GL_QUADS);
-	  glTexCoord2f (tx, ty);   glVertex2i   (qx2, qy2);
-	  glTexCoord2f (0,  ty);   glVertex2i   (qx1, qy2);
-	  glTexCoord2f (0,  0);    glVertex2i   (qx1, qy1);
-	  glTexCoord2f (tx, 0);    glVertex2i   (qx2, qy1);
-	  glEnd ();	
+	  cogl_texture_quad (qx1, qx2, qy1, qy2, 
+			     0,
+			     0,
+			     CLUTTER_FLOAT_TO_FIXED (tx),
+			     CLUTTER_FLOAT_TO_FIXED (ty));
 
 	  lasty += qy2 - qy1;	  
 
@@ -176,8 +172,9 @@ clutter_clone_texture_paint (ClutterActor *self)
 {
   ClutterCloneTexturePrivate  *priv;
   ClutterActor                *parent_texture;
-  gint                         x1, y1, x2, y2;
+  gint                         x_1, y_1, x_2, y_2;
   GLenum                       target_type;
+  ClutterColor                 col = { 0xff, 0xff, 0xff, 0xff };
 
   priv = CLUTTER_CLONE_TEXTURE (self)->priv;
 
@@ -185,6 +182,11 @@ clutter_clone_texture_paint (ClutterActor *self)
   if (!priv->parent_texture)
     return;
 
+  CLUTTER_NOTE (PAINT,
+                "painting clone texture '%s'",
+		clutter_actor_get_name (self) ? clutter_actor_get_name (self)
+                                              : "unknown");
+
   /* parent texture may have been hidden, there for need to make sure its 
    * realised with resources available.  
   */
@@ -192,32 +194,37 @@ clutter_clone_texture_paint (ClutterActor *self)
   if (!CLUTTER_ACTOR_IS_REALIZED (parent_texture))
     clutter_actor_realize (parent_texture);
 
+  cogl_push_matrix ();
+
   /* FIXME: figure out nicer way of getting at this info...  
    */  
   if (clutter_feature_available (CLUTTER_FEATURE_TEXTURE_RECTANGLE) &&
       clutter_texture_is_tiled (CLUTTER_TEXTURE (parent_texture)) == FALSE)
-    target_type = GL_TEXTURE_RECTANGLE_ARB;
+    {
+      target_type = CGL_TEXTURE_RECTANGLE_ARB;
+      cogl_enable (CGL_ENABLE_TEXTURE_RECT|CGL_ENABLE_BLEND);
+    }
   else
-    target_type = GL_TEXTURE_2D;
+    {
+      target_type = CGL_TEXTURE_2D;
+      cogl_enable (CGL_ENABLE_TEXTURE_2D|CGL_ENABLE_BLEND);
+    }
 
-  glEnable (GL_BLEND);
-  glEnable (target_type);
-  glBlendFunc (GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+  col.alpha = clutter_actor_get_opacity (self);
+  cogl_color (&col);
 
-  glColor4ub (255, 255, 255, clutter_actor_get_opacity (self));
-
-  clutter_actor_get_coords (self, &x1, &y1, &x2, &y2);
+  clutter_actor_get_coords (self, &x_1, &y_1, &x_2, &y_2);
 
   CLUTTER_NOTE (PAINT, "paint to x1: %i, y1: %i x2: %i, y2: %i "
 		"opacity: %i",
-		x1, y1, x2, y2,
+		x_1, y_1, x_2, y_2,
 		clutter_actor_get_opacity (self));
 
   /* Parent paint translated us into position */
   clone_texture_render_to_gl_quad (CLUTTER_CLONE_TEXTURE (self), 
-				   0, 0, x2 - x1, y2 - y1);
-  glDisable (target_type);
-  glDisable (GL_BLEND);
+				   0, 0, x_2 - x_1, y_2 - y_1);
+
+  cogl_pop_matrix ();
 }
 
 static void
