@@ -74,7 +74,9 @@ clutter_color_add (const ClutterColor *src1,
  * @dest: return location for the result
  *
  * Subtracts @src2 from @src1 and saves the resulting
- * color inside @dest.
+ * color inside @dest. This function assumes that the components
+ * of @src1 are greater than the components of @src2; the result is,
+ * otherwise, undefined.
  *
  * The alpha channel of @dest is set as the minimum value
  * between the alpha channels of @src1 and @src2.
@@ -88,9 +90,9 @@ clutter_color_subtract (const ClutterColor *src1,
   g_return_if_fail (src2 != NULL);
   g_return_if_fail (dest != NULL);
 
-  dest->red   = CLAMP (src2->red   - src1->red,   0, 255);
-  dest->green = CLAMP (src2->green - src1->green, 0, 255);
-  dest->blue  = CLAMP (src2->blue  - src1->blue,  0, 255);
+  dest->red   = CLAMP (src1->red   - src2->red,   0, 255);
+  dest->green = CLAMP (src1->green - src2->green, 0, 255);
+  dest->blue  = CLAMP (src1->blue  - src2->blue,  0, 255);
 
   dest->alpha = MIN (src1->alpha, src2->alpha);
 }
@@ -128,19 +130,20 @@ clutter_color_darken (const ClutterColor *src,
 }
 
 /**
- * clutter_color_to_hls:
+ * clutter_color_to_hlsx:
  * @src: a #ClutterColor
  * @hue: return location for the hue value or %NULL
  * @luminance: return location for the luminance value or %NULL
  * @saturation: return location for the saturation value or %NULL
  *
- * Converts @src to the HLS format.
+ * Converts @src to the HLS format. Returned hue is in degrees (0 .. 360),
+ * luminance and saturation from interval <0 .. 1>.
  */
 void
-clutter_color_to_hls (const ClutterColor *src,
-		      guint8             *hue,
-		      guint8             *luminance,
-		      guint8             *saturation)
+clutter_color_to_hlsx (const ClutterColor *src,
+		       ClutterFixed       *hue,
+		       ClutterFixed       *luminance,
+		       ClutterFixed       *saturation)
 {
   ClutterFixed red, green, blue;
   ClutterFixed min, max, delta;
@@ -202,7 +205,125 @@ clutter_color_to_hls (const ClutterColor *src,
     }
 
   if (hue)
-    *hue = (guint8) CFX_INT (h * 255);
+    *hue = h;
+
+  if (luminance)
+    *luminance = l;
+
+  if (saturation)
+    *saturation = s;
+}
+
+/**
+ * clutter_color_from_hlsx:
+ * @dest: return location for a #ClutterColor
+ * @hue: hue value (0 .. 360)
+ * @luminance: luminance value (0 .. 1)
+ * @saturation: saturation value (0 .. 1)
+ *
+ * Converts a color expressed in HLS (hue, luminance and saturation)
+ * values into a #ClutterColor.
+ */
+
+void
+clutter_color_from_hlsx (ClutterColor *dest,
+			 ClutterFixed   hue,
+			 ClutterFixed   luminance,
+			 ClutterFixed   saturation)
+{
+  ClutterFixed h, l, s;
+  ClutterFixed m1, m2;
+  
+  g_return_if_fail (dest != NULL);
+
+  l = luminance;
+  s = saturation;
+
+  if (l <= CFX_ONE/2)
+    m2 = CFX_MUL (l, (CFX_ONE + s));
+  else
+    m2 = l + s - CFX_MUL (l,s);
+
+  m1 = 2 * l - m2;
+
+  if (s == 0)
+    {
+      dest->red   = (guint8) CFX_INT (l * 255);
+      dest->green = (guint8) CFX_INT (l * 255);
+      dest->blue  = (guint8) CFX_INT (l * 255);
+    }
+  else
+    {
+      h = hue + CFX_120;
+      while (h > CFX_360)
+	h -= CFX_360;
+      while (h < 0)
+	h += CFX_360;
+
+      if (h < CFX_60)
+	dest->red = (guint8) CFX_INT((m1 + CFX_MUL((m2-m1), h) / 60) * 255);
+      else if (h < CFX_180)
+	dest->red = (guint8) CFX_INT (m2 * 255);
+      else if (h < CFX_240)
+	dest->red = (guint8)CFX_INT((m1+CFX_MUL((m2-m1),(CFX_240-h))/60)*255);
+      else
+	dest->red = (guint8) CFX_INT (m1 * 255);
+
+      h = hue;
+      while (h > CFX_360)
+	h -= CFX_360;
+      while (h < 0)
+	h += CFX_360;
+
+      if (h < CFX_60)
+	dest->green = (guint8)CFX_INT((m1 + CFX_MUL((m2 - m1), h) / 60) * 255);
+      else if (h < CFX_180)
+        dest->green = (guint8) CFX_INT (m2 * 255);
+      else if (h < CFX_240)
+	dest->green =
+	    (guint8) CFX_INT((m1 + CFX_MUL ((m2-m1), (CFX_240-h)) / 60) * 255);
+      else
+	dest->green = (guint8) CFX_INT (m1 * 255);
+
+      h = hue - CFX_120;
+      while (h > CFX_360)
+	h -= CFX_360;
+      while (h < 0)
+	h += CFX_360;
+
+      if (h < CFX_60)
+	dest->blue = (guint8) CFX_INT ((m1 + CFX_MUL ((m2-m1), h) / 60) * 255);
+      else if (h < CFX_180)
+	dest->blue = (guint8) CFX_INT (m2 * 255);
+      else if (h < CFX_240)
+	dest->blue = (guint8)CFX_INT((m1+CFX_MUL((m2-m1),(CFX_240-h))/60)*255);
+      else
+	dest->blue = (guint8) CFX_INT(m1 * 255);
+    }
+}
+
+/**
+ * clutter_color_to_hls:
+ * @src: a #ClutterColor
+ * @hue: return location for the hue value or %NULL
+ * @luminance: return location for the luminance value or %NULL
+ * @saturation: return location for the saturation value or %NULL
+ *
+ * Converts @src to the HLS format. Returned HLS values are from interval
+ * 0 .. 255.
+ */
+void
+clutter_color_to_hls (const ClutterColor *src,
+		      guint8             *hue,
+		      guint8             *luminance,
+		      guint8             *saturation)
+{
+  ClutterFixed h, l, s;
+  
+  clutter_color_to_hlsx (src, &h, &l, &s);
+  
+  if (hue)
+    *hue = (guint8) CFX_INT (h * 255) / 360;
 
   if (luminance)
     *luminance = (guint8) CFX_INT (l * 255);
@@ -229,74 +350,12 @@ clutter_color_from_hls (ClutterColor *dest,
 			guint8        saturation)
 {
   ClutterFixed h, l, s;
-  ClutterFixed m1, m2;
-  
-  g_return_if_fail (dest != NULL);
 
+  h = CLUTTER_INT_TO_FIXED (hue * 360) / 255;
   l = CLUTTER_INT_TO_FIXED (luminance)  / 255;
   s = CLUTTER_INT_TO_FIXED (saturation) / 255;
 
-  if (l <= CFX_ONE/2)
-    m2 = CFX_MUL (l, (CFX_ONE - s));
-  else
-    m2 = l + s - CFX_MUL (l,s);
-
-  m1 = 2 * l - m2;
-
-  if (s == 0)
-    {
-      dest->red   = (guint8) CFX_INT (l * 255);
-      dest->green = (guint8) CFX_INT (l * 255);
-      dest->blue  = (guint8) CFX_INT (l * 255);
-    }
-  else
-    {
-      h = (CLUTTER_INT_TO_FIXED (hue)/ 255) + CFX_120;
-      while (h > CFX_360)
-	h -= CFX_360;
-      while (h < 0)
-	h += CFX_360;
-
-      if (h < CFX_60)
-	dest->red = (guint8) CFX_INT((m1 + CFX_MUL((m2-m1), h) / 60) * 255);
-      else if (h < CFX_180)
-	dest->red = (guint8) CFX_INT (m2 * 255);
-      else if (h < CFX_240)
-	dest->red = (guint8)CFX_INT((m1+CFX_MUL((m2-m1),(CFX_240-h))/60)*255);
-      else
-	dest->red = (guint8) CFX_INT (m1 * 255);
-
-      h = CLUTTER_INT_TO_FIXED (hue) / 255;
-      while (h > CFX_360)
-	h -= CFX_360;
-      while (h < 0)
-	h += CFX_360;
-
-      if (h < CFX_60)
-	dest->green = (guint8)CFX_INT((m1 + CFX_MUL((m2 - m1), h) / 60) * 255);
-      else if (h < CFX_180)
-        dest->green = (guint8) CFX_INT (m2 * 255);
-      else if (h < CFX_240)
-	dest->green =
-	    (guint8) CFX_INT((m1 + CFX_MUL ((m2-m1), (CFX_240-h)) / 60) * 255);
-      else
-	dest->green = (guint8) CFX_INT (m1 * 255);
-
-      h = (CLUTTER_INT_TO_FIXED (hue) / 255) - CFX_120;
-      while (h > CFX_360)
-	h -= CFX_360;
-      while (h < 0)
-	h += CFX_360;
-
-      if (h < CFX_60)
-	dest->blue = (guint8) CFX_INT ((m1 + CFX_MUL ((m2-m1), h) / 60) * 255);
-      else if (h < CFX_180)
-	dest->blue = (guint8) CFX_INT (m2 * 255);
-      else if (h < CFX_240)
-	dest->blue = (guint8)CFX_INT((m1+CFX_MUL((m2-m1),(CFX_240-h))/60)*255);
-      else
-	dest->blue = (guint8) CFX_INT(m1 * 255);
-    }
+  clutter_color_from_hlsx (dest, h, l, s);
 }
 
 /**
@@ -334,33 +393,26 @@ clutter_color_shadex (const ClutterColor *src,
 		      ClutterColor       *dest,
 		      ClutterFixed        shade)
 {
-  guint8 h, l, s;
-  ClutterFixed l1, s1;
+  ClutterFixed h, l, s;
 
   g_return_if_fail (src != NULL);
   g_return_if_fail (dest != NULL);
   
-  clutter_color_to_hls (src, &h, &l, &s);
+  clutter_color_to_hlsx (src, &h, &l, &s);
 
-  l1 = CLUTTER_INT_TO_FIXED (l) / 255;
-  s1 = CLUTTER_INT_TO_FIXED (s) / 255;
+  l = CFX_MUL (l, shade);
+  if (l > CFX_ONE)
+    l = CFX_ONE;
+  else if (l < 0)
+    l = 0;
 
-  l1 = CFX_MUL (l1, shade);
-  if (l1 > CFX_ONE)
-    l1 = CFX_ONE;
-  else if (l1 < 0)
-    l1 = 0;
-
-  s1 = CFX_MUL (s1, shade);
-  if (s1 > CFX_ONE)
-    s1 = CFX_ONE;
-  else if (s1 < 0)
-    s1 = 0;
+  s = CFX_MUL (s, shade);
+  if (s > CFX_ONE)
+    s = CFX_ONE;
+  else if (s < 0)
+    s = 0;
   
-  l = (guint8) CFX_INT (l1 * 255);
-  s = (guint8) CFX_INT (s1 * 255);
-
-  clutter_color_from_hls (dest, h, l, s);
+  clutter_color_from_hlsx (dest, h, l, s);
 }
 
 /**
@@ -397,22 +449,22 @@ clutter_color_from_pixel (ClutterColor *dest,
   dest->red = pixel >> 24;
   dest->green = (pixel >> 16) & 0xff;
   dest->blue = (pixel >> 8) & 0xff;
-  dest->alpha = pixel % 0xff;
+  dest->alpha = pixel & 0xff;
 }
 
 /**
  * clutter_color_parse:
- * @color: a string specifiying a color
+ * @color: a string specifiying a color (named color or #RRGGBBAA)
  * @dest: return location for a #ClutterColor
  *
  * Parses a string definition of a color, filling the
- * <structfield>red</structfield>, <structfield>green</structfield> and
- * <structfield>blue</structfield> channels of @dest. The
- * <structfield>alpha</structfield> channel is not changed. The
- * color in @dest is not allocated.
+ * <structfield>red</structfield>, <structfield>green</structfield>, 
+ * <structfield>blue</structfield> and <structfield>alpha</structfield> 
+ * channels of @dest. If alpha is not specified it will be set full opaque.
+ * The color in @dest is not allocated.
  *
  * The color may be defined by any of the formats understood by
- * <function>XParseColor</function>; these include literal color
+ * <function>pango_color_parse</function>; these include literal color
  * names, like <literal>Red</literal> or <literal>DarkSlateGray</literal>,
  * or hexadecimal specifications like <literal>&num;3050b2</literal> or
  * <literal>&num;333</literal>.
@@ -427,16 +479,43 @@ clutter_color_parse (const gchar  *color,
 {
   PangoColor pango_color;
 
+  /* parse ourselves to get alpha */
+  if (color[0] == '#')
+    {
+      gint32 result;
+
+      if (sscanf (color+1, "%x", &result))
+	{
+	  if (strlen(color) == 9)
+	    {
+	      dest->red   = result >> 24 & 0xff;
+	      dest->green = (result >> 16) & 0xff;
+	      dest->blue  = (result >> 8) & 0xff;
+	      dest->alpha = result & 0xff;
+	      return TRUE;
+	    }
+	  else if (strlen(color) == 7)
+	    {
+	      dest->red   = (result >> 16) & 0xff;
+	      dest->green = (result >> 8) & 0xff;
+	      dest->blue  = result & 0xff;
+	      dest->alpha = 0xff;
+	      return TRUE;
+	    }
+	}
+    }
+  
+  /* Fall back to pango for named colors - note pango does not handle alpha */
   if (pango_color_parse (&pango_color, color))
     {
-      dest->red = pango_color.red;
+      dest->red   = pango_color.red;
       dest->green = pango_color.green;
-      dest->blue = pango_color.blue;
-
+      dest->blue  = pango_color.blue;
+      dest->alpha = 0xff;
       return TRUE;
     }
-  else
-    return FALSE;
+
+  return FALSE;
 }
 
 /**
