@@ -33,16 +33,24 @@
 #include "config.h"
 
 #include "clutter-label.h"
+#include "clutter-layout.h"
 #include "clutter-main.h"
 #include "clutter-enum-types.h"
 #include "clutter-private.h"
 #include "clutter-debug.h"
+#include "clutter-units.h"
 
 #include "pangoclutter.h"
 
 #define DEFAULT_FONT_NAME	"Sans 10"
 
-G_DEFINE_TYPE (ClutterLabel, clutter_label, CLUTTER_TYPE_ACTOR);
+static void clutter_layout_iface_init (ClutterLayoutIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (ClutterLabel,
+                         clutter_label,
+                         CLUTTER_TYPE_ACTOR,
+                         G_IMPLEMENT_INTERFACE (CLUTTER_TYPE_LAYOUT,
+                                                clutter_layout_iface_init));
 
 /* Probably move into main */
 static PangoClutterFontMap  *_font_map = NULL;
@@ -60,6 +68,7 @@ enum
   PROP_WRAP,
   PROP_WRAP_MODE,
   PROP_ELLIPSIZE,
+  PROP_LAYOUT_FLAGS,
 };
 
 #define CLUTTER_LABEL_GET_PRIVATE(obj) \
@@ -71,6 +80,8 @@ struct _ClutterLabelPrivate
   PangoFontDescription *desc;
   
   ClutterColor          fgcol;
+
+  ClutterLayoutFlags    layout_flags;
   
   gchar                *text;
   gchar                *font_name;
@@ -183,6 +194,9 @@ clutter_label_get_property (GObject    *object,
     case PROP_ELLIPSIZE:
       g_value_set_enum (value, priv->ellipsize);
       break;
+    case PROP_LAYOUT_FLAGS:
+      g_value_set_flags (value, priv->layout_flags);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
@@ -190,7 +204,8 @@ clutter_label_get_property (GObject    *object,
 }
 
 static void
-clutter_label_ensure_layout (ClutterLabel *label, gint width)
+clutter_label_ensure_layout (ClutterLabel *label,
+                             gint          width)
 {
   ClutterLabelPrivate  *priv;
 
@@ -209,7 +224,11 @@ clutter_label_ensure_layout (ClutterLabel *label, gint width)
 					      priv->single_line_mode);
       
       pango_layout_set_font_description (priv->layout, priv->desc);
-      pango_layout_set_text (priv->layout, priv->text, -1);
+      
+      if (!priv->use_markup)
+        pango_layout_set_text (priv->layout, priv->text, -1);
+      else
+        pango_layout_set_markup (priv->layout, priv->text, -1);
       
       if (priv->wrap)
 	pango_layout_set_wrap  (priv->layout, priv->wrap_mode);
@@ -251,16 +270,16 @@ clutter_label_paint (ClutterActor *self)
       return;
     }
 
-  clutter_label_ensure_layout (label, clutter_actor_get_width(self));
+  clutter_label_ensure_layout (label, clutter_actor_get_width (self));
 
-  priv->fgcol.alpha   =  clutter_actor_get_opacity(self);
+  priv->fgcol.alpha = clutter_actor_get_opacity(self);
 
   pango_clutter_render_layout (priv->layout, 0, 0, &priv->fgcol, 0);
 }
 
 static void
-clutter_label_allocate_coords (ClutterActor    *self,
-			       ClutterActorBox *box)
+clutter_label_query_coords (ClutterActor        *self,
+			    ClutterActorBox     *box)
 {
   ClutterLabel         *label = CLUTTER_LABEL(self);
   ClutterLabelPrivate  *priv;  
@@ -268,22 +287,22 @@ clutter_label_allocate_coords (ClutterActor    *self,
 
   priv = label->priv;
 
-  clutter_label_ensure_layout (label, box->x2 - box->x1);
+  clutter_label_ensure_layout (label, CLUTTER_UNITS_TO_INT (box->x2 - box->x1));
 
   pango_layout_get_extents (priv->layout, NULL, &logical_rect);
 
-  box->x2 = box->x1 + PANGO_PIXELS (logical_rect.width);
-  box->y2 = box->y1 + PANGO_PIXELS (logical_rect.height);
+  box->x2 = box->x1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.width);
+  box->y2 = box->y1 + CLUTTER_UNITS_FROM_PANGO_UNIT (logical_rect.height);
 
   return;
 }
 
 static void
-clutter_label_request_coords (ClutterActor    *self,
-			      ClutterActorBox *box)
+clutter_label_request_coords (ClutterActor        *self,
+			      ClutterActorBox     *box)
 {
   /* do we need to do anything ? */
-  clutter_label_clear_layout (CLUTTER_LABEL(self));
+  clutter_label_clear_layout (CLUTTER_LABEL (self));
 }
 
 static void 
@@ -327,6 +346,34 @@ clutter_label_finalize (GObject *object)
   G_OBJECT_CLASS (clutter_label_parent_class)->finalize (object);
 }
 
+static ClutterLayoutFlags
+clutter_layout_label_get_layout_flags (ClutterLayout *layout)
+{
+  return CLUTTER_LABEL (layout)->priv->layout_flags;
+}
+
+static void
+clutter_layout_label_height_for_width (ClutterLayout *layout,
+                                       ClutterUnit    width,
+                                       ClutterUnit   *height)
+{
+  ClutterLabel *label = CLUTTER_LABEL (layout);
+  gint layout_height;
+
+  clutter_label_ensure_layout (label, CLUTTER_UNITS_TO_INT (width));
+  pango_layout_get_pixel_size (label->priv->layout, NULL, &layout_height);
+
+  if (height)
+    *height = CLUTTER_UNITS_FROM_INT (layout_height);
+}
+
+static void
+clutter_layout_iface_init (ClutterLayoutIface *iface)
+{
+  iface->get_layout_flags = clutter_layout_label_get_layout_flags;
+  iface->height_for_width = clutter_layout_label_height_for_width;
+}
+
 static void
 clutter_label_class_init (ClutterLabelClass *klass)
 {
@@ -335,7 +382,7 @@ clutter_label_class_init (ClutterLabelClass *klass)
 
   actor_class->paint           = clutter_label_paint;
   actor_class->request_coords  = clutter_label_request_coords;
-  actor_class->allocate_coords = clutter_label_allocate_coords;
+  actor_class->query_coords    = clutter_label_query_coords;
 
   gobject_class->finalize   = clutter_label_finalize;
   gobject_class->dispose    = clutter_label_dispose;
@@ -421,6 +468,10 @@ clutter_label_class_init (ClutterLabelClass *klass)
 			 PANGO_ALIGN_LEFT,
 			 CLUTTER_PARAM_READWRITE));
 
+  g_object_class_override_property (gobject_class,
+                                    PROP_LAYOUT_FLAGS,
+                                    "layout-flags");
+
   g_type_class_add_private (gobject_class, sizeof (ClutterLabelPrivate));
 }
 
@@ -431,10 +482,18 @@ clutter_label_init (ClutterLabel *self)
 
   self->priv = priv = CLUTTER_LABEL_GET_PRIVATE (self);
 
-  if (_context == NULL)
+  if (G_UNLIKELY (_context == NULL))
     {
+      ClutterBackend *backend;
+      gdouble resolution;
+
+      backend = clutter_get_default_backend ();
+      resolution = clutter_backend_get_resolution (backend);
+      if (resolution < 0)
+        resolution = 96.0;
+
       _font_map = PANGO_CLUTTER_FONT_MAP (pango_clutter_font_map_new ());
-      /* pango_clutter_font_map_set_resolution (font_map, 96.0, 96.0); */
+      pango_clutter_font_map_set_resolution (_font_map, resolution);
       _context = pango_clutter_font_map_create_context (_font_map);
     }
 
@@ -447,6 +506,7 @@ clutter_label_init (ClutterLabel *self)
   priv->layout        = NULL;
   priv->text          = NULL;
   priv->attrs         = NULL;
+  priv->layout_flags  = CLUTTER_LAYOUT_HEIGHT_FOR_WIDTH;
 
   priv->fgcol.red     = 0;
   priv->fgcol.green   = 0;
@@ -988,7 +1048,7 @@ clutter_label_get_attributes (ClutterLabel *label)
  *
  * Sets whether the text of the label contains markup in <link
  * linkend="PangoMarkupFormat">Pango's text markup
- * language</link>. See clutter_label_set_markup().
+ * language</link>.
  **/
 void
 clutter_label_set_use_markup (ClutterLabel *label,
@@ -1075,7 +1135,7 @@ clutter_label_set_alignment (ClutterLabel   *label,
  *
  * Since 0.2
  **/
-gboolean
+PangoAlignment
 clutter_label_get_alignment (ClutterLabel *label)
 {
   g_return_val_if_fail (CLUTTER_IS_LABEL (label), FALSE);
