@@ -38,12 +38,31 @@ G_BEGIN_DECLS
 #define CLUTTER_IS_TEXTURE_CLASS(klass) (G_TYPE_CHECK_CLASS_TYPE ((klass), CLUTTER_TYPE_TEXTURE))
 #define CLUTTER_TEXTURE_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS ((obj), CLUTTER_TYPE_TEXTURE, ClutterTextureClass))
 
+/**
+ * ClutterTextureError:
+ * @CLUTTER_TEXTURE_ERROR_OUT_OF_MEMORY: OOM condition
+ * @CLUTTER_TEXTURE_ERROR_NO_YUV: YUV operation attempted but no YUV support
+ *   found
+ *
+ * Error enumeration for #ClutterTexture
+ *
+ * Since: 0.4
+ */
+typedef enum {
+  CLUTTER_TEXTURE_ERROR_OUT_OF_MEMORY,
+  CLUTTER_TEXTURE_ERROR_NO_YUV
+} ClutterTextureError;
+
+#define CLUTTER_TEXTURE_ERROR   (clutter_texture_error_quark ())
+GQuark clutter_texture_error_quark (void);
+
 typedef struct _ClutterTexture        ClutterTexture;
 typedef struct _ClutterTextureClass   ClutterTextureClass;
 typedef struct _ClutterTexturePrivate ClutterTexturePrivate;
 
 struct _ClutterTexture
 {
+  /*< private >*/
   ClutterActor         parent;
 
   ClutterTexturePrivate *priv;
@@ -58,6 +77,7 @@ struct _ClutterTextureClass
 		         gint            height);
   void (*pixbuf_change) (ClutterTexture *texture);
 
+  /*< private >*/
   /* padding, for future expansion */
   void (*_clutter_texture1) (void);
   void (*_clutter_texture2) (void);
@@ -67,37 +87,65 @@ struct _ClutterTextureClass
   void (*_clutter_texture6) (void);
 };
 
+/**
+ * ClutterTextureFlags:
+ * @CLUTTER_TEXTURE_RGB_FLAG_BGR: FIXME
+ * @CLUTTER_TEXTURE_RGB_FLAG_PREMULT: FIXME
+ * @CLUTTER_TEXTURE_YUV_FLAG_YUV2: FIXME
+ *
+ * Flags for clutter_texture_set_from_rgb_data() and
+ * clutter_texture_set_from_rgb_data().
+ *
+ * Since: 0.4
+ */
+typedef enum { /*< prefix=CLUTTER_TEXTURE >*/
+    CLUTTER_TEXTURE_RGB_FLAG_BGR     = 1 << 1,
+    CLUTTER_TEXTURE_RGB_FLAG_PREMULT = 1 << 2, /* FIXME: not handled */
+    CLUTTER_TEXTURE_YUV_FLAG_YUV2    = 1 << 3
+
+    /* FIXME: add compressed types ? */
+} ClutterTextureFlags;
+
 GType clutter_texture_get_type (void) G_GNUC_CONST;
 
-ClutterActor *clutter_texture_new             (void);
-ClutterActor *clutter_texture_new_from_pixbuf (GdkPixbuf      *pixbuf);
-void          clutter_texture_set_from_data   (ClutterTexture *texture,
-                                               const guchar   *data,
-                                               gboolean        has_alpha,
-                                               gint            width,
-                                               gint            height,
-                                               gint            rowstride,
-                                               gint            bpp);
-void          clutter_texture_set_pixbuf      (ClutterTexture *texture,
-                                               GdkPixbuf      *pixbuf);
-GdkPixbuf *   clutter_texture_get_pixbuf      (ClutterTexture *texture);
-void          clutter_texture_get_base_size   (ClutterTexture *texture,
-                                               gint           *width,
-                                               gint           *height);
+ClutterActor *clutter_texture_new                 (void);
+ClutterActor *clutter_texture_new_from_pixbuf     (GdkPixbuf      *pixbuf);
+gboolean      clutter_texture_set_from_rgb_data   (ClutterTexture *texture,
+						   const guchar   *data,
+						   gboolean        has_alpha,
+						   gint            width,
+						   gint            height,
+						   gint            rowstride,
+						   gint            bpp,
+						   ClutterTextureFlags  flags,
+						   GError        **error);
+gboolean      clutter_texture_set_from_yuv_data   (ClutterTexture *texture,
+						   const guchar   *data,
+						   gint            width,
+						   gint            height,
+						   ClutterTextureFlags  flags,
+						   GError        **error);
+gboolean      clutter_texture_set_pixbuf          (ClutterTexture *texture,
+						   GdkPixbuf      *pixbuf,
+						   GError        **error);
+GdkPixbuf *   clutter_texture_get_pixbuf          (ClutterTexture *texture);
+void          clutter_texture_get_base_size       (ClutterTexture *texture,
+                                                   gint           *width,
+                                                   gint           *height);
 
 /* Below mainly for subclassed texture based actors */
 
-void clutter_texture_bind_tile               (ClutterTexture *texture,
-                                              gint            index);
-void clutter_texture_get_n_tiles             (ClutterTexture *texture,
+void     clutter_texture_bind_tile           (ClutterTexture *texture,
+                                              gint            index_);
+void     clutter_texture_get_n_tiles         (ClutterTexture *texture,
                                               gint           *n_x_tiles,
                                               gint           *n_y_tiles);
-void clutter_texture_get_x_tile_detail       (ClutterTexture *texture, 
+void     clutter_texture_get_x_tile_detail   (ClutterTexture *texture, 
 				              gint            x_index,
 				              gint           *pos,
 				              gint           *size,
 				              gint           *waste);
-void clutter_texture_get_y_tile_detail       (ClutterTexture *texture, 
+void     clutter_texture_get_y_tile_detail   (ClutterTexture *texture, 
 				              gint            y_index,
 				              gint           *pos,
 				              gint           *size,
