@@ -61,14 +61,16 @@ typedef struct _ClutterTimelinePrivate ClutterTimelinePrivate;
 struct _ClutterTimeline
 {
   /*< private >*/
-  GObject                 parent;
+  GObject parent;
   ClutterTimelinePrivate *priv;
 };
 
 struct _ClutterTimelineClass
 {
+  /*< private >*/
   GObjectClass parent_class;
   
+  /*< public >*/
   void (*started)   (ClutterTimeline *timeline);
   void (*completed) (ClutterTimeline *timeline);
   void (*paused)    (ClutterTimeline *timeline);
@@ -76,6 +78,7 @@ struct _ClutterTimelineClass
   void (*new_frame) (ClutterTimeline *timeline,
 		     gint             frame_num);
 
+  /*< private >*/
   void (*_clutter_timeline_1) (void);
   void (*_clutter_timeline_2) (void);
   void (*_clutter_timeline_3) (void);
@@ -87,6 +90,8 @@ GType clutter_timeline_get_type (void) G_GNUC_CONST;
 
 ClutterTimeline *clutter_timeline_new               (guint            n_frames,
                                                      guint            fps);
+ClutterTimeline *clutter_timeline_clone             (ClutterTimeline *timeline);
+
 guint            clutter_timeline_get_speed         (ClutterTimeline *timeline);
 void             clutter_timeline_set_speed         (ClutterTimeline *timeline,
                                                      guint            fps);
@@ -106,6 +111,9 @@ void             clutter_timeline_set_n_frames      (ClutterTimeline *timeline,
                                                      guint            n_frames);
 guint            clutter_timeline_get_n_frames      (ClutterTimeline *timeline);
 gboolean         clutter_timeline_is_playing        (ClutterTimeline *timeline);
+void             clutter_timeline_set_delay         (ClutterTimeline *timeline,
+                                                     guint            msecs);
+guint            clutter_timeline_get_delay         (ClutterTimeline *timeline);
 
 G_END_DECLS
 
