@@ -34,43 +34,6 @@
 #include "clutter-util.h"
 #include "clutter-main.h"
 
-static int TrappedErrorCode = 0;
-static int (*old_error_handler) (Display *, XErrorEvent *);
-
-static int
-error_handler(Display     *xdpy,
-	      XErrorEvent *error)
-{
-  TrappedErrorCode = error->error_code;
-  return 0;
-}
-
-/**
- * clutter_util_trap_x_errors:
- *
- * Trap X errors so they don't cause an abort.
- */
-void
-clutter_util_trap_x_errors(void)
-{
-  TrappedErrorCode  = 0;
-  old_error_handler = XSetErrorHandler(error_handler);
-}
-
-/**
- * clutter_util_untrap_x_errors:
- *
- * Stop trapping X errors.
- *
- * Return value: 0 if there was no error, or the last X error that occurred.
- */
-int
-clutter_util_untrap_x_errors(void)
-{
-  XSetErrorHandler(old_error_handler);
-  return TrappedErrorCode;
-}
-
 /**
  * clutter_util_next_p2:
  * @a: Value to get the next power
